<?php

namespace FiloBlu\Rma\Plugin\Returns;

use FiloBlu\Rma\Helper\RmaHelper;
use FiloBlu\Rma\Helper\ChangeSizeAttributeHelper;
use Psr\Log\LoggerInterface;
use Magento\Rma\Controller\Returns\Create as ReturnsCreate;
use Magento\Framework\Stdlib\Cookie\PublicCookieMetadataFactory;
use Magento\Framework\Stdlib\CookieManagerInterface;


class Create
{
    /**
     * @var RmaHelper
     */
    private $rmaHelper;
    /**
     * @var ChangeSizeAttributeHelper
     */
    private $changeSizeAttributeHelper;

    /**
     * @var PublicCookieMetadataFactory
     */
    private $publicCookieMetadataFactory;
    /**
     * @var CookieManagerInterface
     */
    private $cookieManager;
    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * Create constructor.
     * @param RmaHelper $rmaHelper
     * @param ChangeSizeAttributeHelper $changeSizeAttributeHelper
     * @param PublicCookieMetadataFactory $publicCookieMetadataFactory
     * @param CookieManagerInterface $cookieManager
     * @param LoggerInterface $logger
     */
    public function __construct(
        RmaHelper $rmaHelper,
        ChangeSizeAttributeHelper $changeSizeAttributeHelper,
        PublicCookieMetadataFactory $publicCookieMetadataFactory,
        CookieManagerInterface $cookieManager,
        LoggerInterface $logger)
    {
        $this->rmaHelper = $rmaHelper;
        $this->changeSizeAttributeHelper = $changeSizeAttributeHelper;
        $this->publicCookieMetadataFactory = $publicCookieMetadataFactory;
        $this->cookieManager = $cookieManager;
        $this->logger = $logger;
    }

    /**
     * @param ReturnsCreate $subject
     * @param callable $proceed
     * @return mixed
     */
    public function aroundExecute(ReturnsCreate $subject, callable $proceed)
    {
        try {
            if ($subject->getRequest()->getMethod() != "POST") {
                return $proceed();
            }
            $post = $subject->getRequest()->getPostValue();
            $isValid = true;
            $error = [];
            if (isset($post['items'])) {

                $changeSizeResolution = $this->changeSizeAttributeHelper->getChangeSizeAttributeOptionId();
                foreach ($post['items'] as $item) {

                    if (isset($item['resolution']) && $item['resolution'] == $changeSizeResolution) {
                        if (!isset($item['reason'])) {
                            $isValid = false;
                            $error [] = "Reason is empty";
                        } else {
                            $isValid = $this->rmaHelper->checkIfAllowedMagentoChangeSizeReason($item['reason']);
                            if (!$isValid) {
                                $error [] = "The selected reason isn't allowed with selected resolution";
                            }
                            if (!isset($item['change_size']) || (isset($item['change_size']) && $item['change_size'] == 0)) {
                                $isValid = false;
                                $error [] = "Change Size isn't setted properly";
                            }
                        }
                    }
                    if (!$isValid) {
                        if (count($error) > 0) {
                            $details = implode(",", $error);
                        } else {
                            $details = "";
                        }
                        $message = __('We can\'t create a return transaction. Errors:' . $details);
                        $metadata = $this->publicCookieMetadataFactory->create()->setPath('/')->setSecure(false)->setHttpOnly(false);
                        $this->cookieManager->setPublicCookie('mage-messages', json_encode(['type' => 'error', 'text' => $message]), $metadata);
                        $subject->getRequest()->setPostValue([]);
                        return $proceed();
                    } else {
                        return $proceed();
                    }
                }
            }
            return $proceed();
        } catch (\Exception $e) {
            $subject->getRequest()->setPostValue([]);
            $this->logger->critical($e->getMessage());
            return $proceed();
        } catch (\Throwable $t) {
            $subject->getRequest()->setPostValue([]);
            $this->logger->critical($t->getMessage());
            return $proceed();
        }
    }
}